*************************************************
DEFINE CLASS CProgrammaticClassParser AS SESSION OLEPUBLIC
*************************************************
	PROTECTED nClassStartElement, nClassEndElement, ;
		nLastClassCharCount, aClassFile[1], Version, ;
		cErrorString
	
	Version               = "1.1"
	nClassStartElement    = 1
	nClassEndElement      = 1
	nLastClassCharCount   = 1
	cErrorString          = ""
	DIMENSION aClassFile[1]

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  Init()
	*-------------------------------------------------------
	*) Description: 
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE Init()
		SET DELETED ON
		THIS.CreateClassCursors()
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  CreateClassCursors()
	*-------------------------------------------------------
	*) Description:  Builds the required cursors 
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE CreateClassCursors()
		
		IF !USED('cn_classes')
			CREATE CURSOR "cn_classes" (id C(10), filenameex C(240), classname c(240), ;
				parent c(240), olepublic L, startchar N(8))
		
			INDEX ON id         					TAG id OF cn_classes
			INDEX ON filenameex 					TAG filenameex OF cn_classes
			INDEX ON ALLTRIM(UPPER(classname))  	TAG classname OF cn_classes
			INDEX ON ALLTRIM(UPPER(parent))     	TAG parent OF cn_classes
		ENDIF

		IF !USED('cn_properties')
			CREATE CURSOR "cn_properties" (id C(10), classid C(10), propname C(240), value M, ;
				Protected L, Hidden L, startchar N(8))

			INDEX ON classid  TAG classid OF cn_properties
			INDEX ON propname TAG propname OF cn_properties
		ENDIF

		IF !USED('cn_methods')
			CREATE CURSOR "cn_methods" (id C(10), classid C(10), methname C(240), parameters M, ;
				Protected L, Hidden L, startchar N(8))

			INDEX ON id 		TAG id 			OF cn_methods
			INDEX ON classid 	TAG classid 	OF cn_methods
			INDEX ON methname 	TAG methname 	OF cn_methods
		ENDIF
		
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  BuildClassesFromFile()
	*-------------------------------------------------------
	*) Description:  Accepts a filename. Stores the file
	*)	in an array and iterates through it pulling out all 
	*)	class information, properties, methods, etc.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE BuildClassesFromFile(tcFullFilename)
		LOCAL llRetVal, lcString, lnStartChar, laClass[1], ;
			laProperties[1], laMethods[1], lnKey 
		
		llRetVal = THIS.StoreFileInFileArray(tcFullFilename)
		IF llRetVal
			DO WHILE THIS.FindNextClassInFileArray(@laClass)
				lcKey = THIS.AddClass(tcFullFilename, @laClass)
					IF THIS.FindPropertiesInClass(@laProperties)
						THIS.AddProperties(lcKey, @laProperties)
					ENDIF

					IF THIS.FindMethodsInClass(@laMethods)
						THIS.AddMethods(lcKey, @laMethods)
					ENDIF
			ENDDO
		ENDIF		
		
		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  DestroyClassesFromFile()
	*-------------------------------------------------------
	*) Description:  Removes all class information that
	*)	was found in tcFullFilename
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE DestroyClassesFromFile(tcFullFilename)
		LOCAL llRetVal, lcFullFilename, lcClassID
		
		lcFullFilename = ALLTRIM(tcFullFilename)

		DO WHILE SEEK(lcFullFilename, "cn_classes", "filenameex")
			lcClassID = cn_classes.id
			
			SELECT cn_classes
			DELETE ALL FOR cn_classes.id = lcClassID
			
			SELECT cn_properties
			DELETE ALL FOR cn_properties.classid = lcClassID
			
			SELECT cn_methods
			DELETE ALL FOR cn_methods.classid = lcClassID
		ENDDO

	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  StoreFileInFileArray()
	*-------------------------------------------------------
	*) Description:  Stores the contents of a file in
	*)	the aClassFile array. Each line in the file will
	*)	be stored in a separate row of the array.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE StoreFileInFileArray(tcFullFilename)
		LOCAL llRetVal, lcFileContents, lcOldError
		
		llRetVal = IIF(FILE(tcFullFilename), .T., .F.)
		IF llRetVal
			
			*-- If the file is already open or otherwise
			*-- unavailable, capture the error here
			*-- and add the error string to the
			*-- cErrorString property
			lcOldError = ON("ERROR")
			ON ERROR llRetVal = .F.
			lcFileContents = FILETOSTR(tcFullFilename)
			IF !llRetVal
				THIS.cErrorString = MESSAGE() + CHR(13) + ALLTRIM(tcFullFileName)
			ELSE
				lcFileContents = STRTRAN(lcFileContents, CHR(9), " ")
				= ALINES(this.aClassFile, lcFileContents)

				THIS.nClassStartElement = 1
				THIS.nClassEndElement   = 1
			ENDIF
			ON ERROR &lcOldError
		ENDIF
		
		RETURN llRetVal
		
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  AddClass()
	*-------------------------------------------------------
	*) Description:  Adds a class to the class cursor
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Character
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE AddClass(tcFullFilename, raClass)
		LOCAL lcRetVal, lcFileContents
		
		lcRetVal = ""
		lcRetVal = SYS(2015)
		INSERT INTO "cn_classes" (id, filenameex, classname, parent, olepublic, startchar) ;
			VALUES (lcRetVal, tcFullFilename, raClass[1], raClass[2], raClass[3], raClass[4])
		
		RETURN lcRetVal

	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  AddProperties()
	*-------------------------------------------------------
	*) Description:  Adds properties to the properties cursor
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Character
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE AddProperties(tcClassKey, raProperties)
		LOCAL lcFileContents, lnLoop, lcID
		
		FOR lnLoop = 1 to ALEN(raProperties,1)
	
			lcID = SYS(2015)

			INSERT INTO "cn_properties" (id, classid, propname, protected, hidden, value) ;
			VALUES (lcID, tcClassKey, raProperties[lnLoop,1], raProperties[lnLoop,2], raProperties[lnLoop,3], raProperties[lnLoop,4])

		ENDFOR

		RETURN 

	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  AddMethods()
	*-------------------------------------------------------
	*) Description:  Adds methods to the methods cursor
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE AddMethods(tcClassKey, raMethods)
		LOCAL lcFileContents, lnLoop, lcID
		
		FOR lnLoop = 1 to ALEN(raMethods,1)

			lcID = SYS(2015)

			INSERT INTO "cn_methods" (id, classid, methname, protected, hidden, parameters, startchar) ;
			VALUES (lcID, tcClassKey, raMethods[lnLoop,1], raMethods[lnLoop,2], raMethods[lnLoop,3], raMethods[lnLoop,4], raMethods[lnLoop,5])

		ENDFOR
		RETURN 

	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  FindNextClassInFileArray()
	*-------------------------------------------------------
	*) Description:  Finds the next class in the aClassFile
	*	array
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE FindNextClassInFileArray(raClass)
		LOCAL llRetVal, lcSearchFor, lcSearchFor2, ;
			lnCount, lnWords, laClassDef[1]
		
		llRetVal = .F.
		
		lcSearchFor = "DEFINE CLASS "
		DIMENSION raClass[4]
		
		*-- Scan through each element looking for a "DEFINE CLASS"
		*-- make sure the expression appears at the beginning of the
		*-- element to make sure it is really a class definition and
		*-- not a commented definition or just the words appearing
		*-- in a comment.
		FOR lnCount	= THIS.nClassStartElement TO ALEN(THIS.aClassFile)
			IF LEFT(UPPER(THIS.aClassFile[lnCount]), LEN(lcSearchFor)) == lcSearchFor
				*-- We have the line of a class definition. Now we
				*-- need to pull out the class name, the parent class
				*-- name and whether is it OLEPUBLIC. Scan through
				*-- each character looking for spaces.

				llRetVal = .T.
				THIS.nClassStartElement = lnCount + 1
				
				lnWords = THIS.WordsToArray(THIS.aClassFile[lnCount], @laClassDef)
				
				*-- Each word in the line is now an element in the array. The
				*-- name of the class will be the 3rd element. The parent name
				*-- will be the 5th element. If there is a 6th element it
				*-- could be the OLEPUBLIC attribute. For this to be a valid
				*-- class definition there must be at least 5 words "DEFINE
				*-- CLASS XXXX AS XXXX"
				
				IF lnWords >= 5
					raClass[1] = laClassDef[3]
					raClass[2] = laClassDef[5]
					raClass[3] = IIF(lnWords > 5 AND UPPER(laClassDef[6]) = "OLEPUBLIC", .T., .F.)
					raClass[4] = THIS.LineNumberToCharNumber(lnCount)
				ENDIF
				
				*-- Find the line that the class definition ends. This could be an
				*-- ENDDEFINE or another DEFINE CLASS or the end of the file.
				lcSearchFor2 = "ENDDEFINE"
				FOR lnCount	= THIS.nClassStartElement TO ALEN(THIS.aClassFile)
	
					THIS.nLastClassCharCount = THIS.nLastClassCharCount + LEN(THIS.aClassFile[lnCount])		

					IF LEFT(UPPER(THIS.aClassFile[lnCount]), LEN(lcSearchFor)) == lcSearchFor OR ;
						LEFT(UPPER(THIS.aClassFile[lnCount]), LEN(lcSearchFor2)) == lcSearchFor2 OR ;
						lnCount = ALEN(THIS.aClassFile)
							THIS.nClassEndElement = lnCount
							EXIT
					ENDIF
				ENDFOR
				
				EXIT
			ELSE
				THIS.nLastClassCharCount = THIS.nLastClassCharCount + LEN(THIS.aClassFile[lnCount])		
			ENDIF
		ENDFOR

			RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  FindPropertiesInClass()
	*-------------------------------------------------------
	*) Description:  Finds all the properties in the 
	*)	class pointed to by nClassStartElement and 
	*)	nClassEndElement
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE FindPropertiesInClass(raProperties)
		LOCAL llRetVal
		
		*-- Reset the raProperties array
		DIMENSION raProperties[1,5]
		STORE .F. to raProperties
		
		llRetVal = THIS.FindProtectedPropertiesInClass(@raProperties)
		llRetVal = THIS.FindFreePropertiesInClass(@raProperties) OR llRetVal 
		
		RETURN llRetVal
	ENDPROC
	
		
	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  FindProtectedPropertiesInClass()
	*-------------------------------------------------------
	*) Description:  Finds all the protected properties in 
	*)	the class pointed to by nClassStartElement and 
	*)	nClassEndElement
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE FindProtectedPropertiesInClass(raProperties)
		LOCAL llRetVal, lnCount, lnWords, laWords[1], ;
			llProtected, llHidden, lcString, ;
			laProperties[1], llContinueLine, lnLoop, ;
			lnProperties, lnPropCount

		llRetVal = .F.
		lnProperties = 0
		lnPropCount = 1
		lnLoopCount = 0
		
		*-- Scan through each element looking for a PROTECTED or
		*-- HIDDEN keyword. There can be multiple lines with these
		*-- keywords butthey will all appear before the first 
		*-- function or method. Also, a distinction has to be made 
		*-- between PROTECTED/HIDDEN properties and functions.
			FOR lnCount	= THIS.nClassStartElement TO THIS.nClassEndElement
				lnWords = THIS.WordsToArray(THIS.aClassFile[lnCount], @laWords)
		
			
				*-- If we see a procedure or function then we need to stop
				*-- looking for protected/hidden properties. So, check for
				*-- PROCEDURE, PROTECTED/HIDDEN PROCEDURE, FUNCTION, 
				*-- PROTECTED/HIDDEN FUNCTION
				IF UPPER(laWords[1]) = "PROCEDURE" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "PROCEDURE") OR ;
					UPPER(laWords[1]) = "FUNCTION" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "FUNCTION")
				
					EXIT
				ENDIF
				
				*-- If it's less than 2 words it can't be a protected
				*-- property.
				IF lnWords < 2
					LOOP
				ENDIF
				
				*-- If it doesn't have a PROTECTED/HIDDEN keyword in
				*-- the beginning move to the next line.
				IF laWords[1] # "PROTECTED" AND laWords[1] # "HIDDEN"
					LOOP
				ENDIF

				*-- If we are here, we know that the line of code is not a
				*-- procedure or function and the first word is either
				*-- PROTECTED or HIDDEN. It must be a property list.
				llRetVal = .T.
				llProtected = IIF(UPPER(laWords[1]) = "PROTECTED", .T., .F.)	
				llHidden    = IIF(UPPER(laWords[1]) = "HIDDEN", .T., .F.)	

				*-- Remove the protected or hidden keyword from the line 
				*-- and change the list to an array.
				IF llProtected
					lcString = SUBSTR(ALLTRIM(THIS.aClassFile[lnCount]), LEN("PROTECTED "))
				ELSE && llHidden
					lcString = SUBSTR(ALLTRIM(THIS.aClassFile[lnCount]), LEN("HIDDEN "))
				ENDIF
				lnWords  = THIS.ListToArray(lcString, @laProperties)
				
				*-- Redimension the raProperties array. If the last "word"
				*-- in the laProperties array is a ";" dimension one less
				llContinueLine = IIF(laProperties[lnWords] = ";", .T., .F.)
				IF llContinueLine
					lnProperties = lnProperties + lnWords - 1
				ELSE
					lnProperties = lnProperties + lnWords
				ENDIF

				*-- Scan through the array adding each property to the
				*-- raProperties array 
				lnLoopCount = 0
				DIMENSION raProperties[lnProperties, 5]
				FOR lnLoop = lnPropCount TO lnProperties
					lnLoopCount = lnLoopCount + 1
					raProperties[lnLoop,1] = laProperties[lnLoopCount]
					raProperties[lnLoop,2] = llProtected
					raProperties[lnLoop,3] = llHidden
					raProperties[lnLoop,4] = ".F."
					raProperties[lnLoop,5] = lnCount
				ENDFOR
				lnPropCount = ALEN(raProperties,1) + 1
				
				*-- If llContinueLine is true then we have multiple lines
				*-- of PROTECTED/HIDDEN properties.
				DO WHILE llContinueLine					
					lnCount = lnCount + 1
					lnPropCount = ALEN(raProperties,1) + 1
					lcString = THIS.aClassFile[lnCount]
					lnWords  = THIS.ListToArray(lcString, @laProperties)
					llContinueLine = IIF(laProperties[lnWords] = ";", .T., .F.)
					IF llContinueLine
						lnProperties = lnProperties + lnWords - 1 
					ELSE
						lnProperties = lnProperties + lnWords 
					ENDIF
					DIMENSION raProperties[lnProperties, 5]

					*-- Scan through the array adding each property to the
					*-- raProperties array
					lnWordCount = 0
					FOR lnLoop = lnPropCount TO lnProperties
						lnWordCount = lnWordCount + 1
						raProperties[lnLoop,1] = laProperties[lnWordCount]
						raProperties[lnLoop,2] = llProtected
						raProperties[lnLoop,3] = llHidden
						raProperties[lnLoop,4] = ".F."
						raProperties[lnLoop,5] = lnCount
					ENDFOR
				ENDDO
			ENDFOR

		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  FindFreePropertiesInClass()
	*-------------------------------------------------------
	*) Description:  Finds "free" properties in the class
	*)	pointed to by nClassStartElement and nClassEndElement.
	*)	A free property is a property that appears somewhere
	*)	in the class header (between the beginning of the class 
	*)	definition and either the end of the class or the first 
	*)	method/function, whichever comes first. Basically, 
	*)	any line in the class header that has an "=" in it
	*)	is a property definition.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE FindFreePropertiesInClass(raProperties)
		LOCAL llRetVal, lnCount, lnWords, laWords[1], lcProperty, ;
			lnElement, lnRow, lnRowCount
		
		llRetVal = .F.
		*-- Scan through each element looking for an
		*-- "=". Everything to the left of the = is
		*-- the property name, everything to the 
		*-- right is the property default value.
		FOR lnCount	= THIS.nClassStartElement TO THIS.nClassEndElement
			lnWords = THIS.WordsToArray(THIS.aClassFile[lnCount], @laWords)
	
		
			*-- If we see a procedure or function then we need to stop
			*-- looking for properties since they can only appear in 
			*-- the class header.
			IF UPPER(laWords[1]) = "PROCEDURE" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "PROCEDURE") OR ;
				UPPER(laWords[1]) = "FUNCTION" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "FUNCTION")
			
				EXIT
			ENDIF
		
			*-- If the first words is a comment or the second word is 
			*-- not an = it can't be a property
			IF lnWords < 3 OR LEFT(laWords[1],1) = "*" OR laWords[2] # "="
				LOOP
			ENDIF

			*-- We have a weener. See if this property is already in the
			*-- raProperties array. If it is, add the property value to 
			*-- the existing element. If it isn't, add a new row to
			*-- the array and add the info.
			llRetVal = .T.
			lnRow = 0
			lnElement = ASCAN(raProperties, laWords[1])
			IF lnElement > 0
				lnRow = ASUBSCRIPT(raProperties, lnElement, 1)
			ENDIF
			
			IF lnRow > 0
				raProperties[lnRow,4] = laWords[3]
			ELSE
				lnRowCount = IIF(VARTYPE(raProperties[1,1]) # "C", 1, ALEN(raProperties,1) + 1)
				DIMENSION raProperties[lnRowCount, 5]
				raProperties[lnRowCount, 1] = laWords[1]
				raProperties[lnRowCount, 2] = .F.
				raProperties[lnRowCount, 3] = .F.
				raProperties[lnRowCount, 4] = laWords[3]
			ENDIF
		ENDFOR
		
		RETURN llRetVal	
	ENDPROC
	
		*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  FindMethodsInClass()
	*-------------------------------------------------------
	*) Description:  Finds all the procedures/functions
	*)	in the class pointed to by nClassStartElement and 
	*)	nClassEndElement
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE FindMethodsInClass(raMethods)
		LOCAL llRetVal, lnCount, lnWords, ;
			laWords[1], llProtected, llHidden, lcString, ;
			laProperties[1], llContinueLine, lnLoop, ;
			lnProperties, laMethods[1], lnMethods

		llRetVal  = .F.
		lnMethods = 0
		
		*-- Scan through each element looking for a procedure
		*-- of function
		FOR lnCount	= THIS.nClassStartElement TO THIS.nClassEndElement
			lnWords = THIS.WordsToArray(THIS.aClassFile[lnCount], @laWords)
	
		
			*-- PROCEDURE, PROTECTED/HIDDEN PROCEDURE, FUNCTION, 
			*-- PROTECTED/HIDDEN FUNCTION
			IF NOT (UPPER(laWords[1]) = "PROCEDURE" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "PROCEDURE") OR ;
				UPPER(laWords[1]) = "FUNCTION" OR (lnWords > 1 AND LEFT(laWords[1],1) # '*' AND UPPER(laWords[2]) = "FUNCTION"))
			
				LOOP
			ENDIF
			
			*-- It's a boy!
			llRetVal 	= .T.
			lnMethods 	= lnMethods + 1
			llProtected = IIF(UPPER(laWords[1]) = "PROTECTED", .T., .F.)	
			llHidden    = IIF(UPPER(laWords[1]) = "HIDDEN", .T., .F.)	
			
			DIMENSION raMethods[lnMethods,5]
			IF llHidden OR llProtected
				raMethods[lnMethods,1] = laWords[3]
			ELSE
				raMethods[lnMethods,1] = laWords[2]
			ENDIF
			raMethods[lnMethods,2] = llProtected
			raMethods[lnMethods,3] = llHidden
			raMethods[lnMethods,4] = ""
			raMethods[lnMethods,5] = THIS.LineNumberToCharNumber(lnCount)
		ENDFOR			

		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetNthGenerationClasses()
	*-------------------------------------------------------
	*) Description: Returns an array of nth generation classes.
	*)	Examples of nth generation: 1st generation classes are
	*)	root classes (classes that have no higher parents),
	*)	2nd generation classes are children of 1st generation
	*)	classes, 3rd generation classes are grand-children of 
	*)	1st generation classes, etc. Optionally, if 
	*)	tcFullFilename is passed only classes from a particular 
	*)	file will be returned.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetNthGenerationClasses(tnLevel, raChildClasses, tcFullFilename)
		LOCAL lnRetVal, lnRecNo, lnLoop, lnLevel, ;
			lcParent, lcParentID, lcScanFor, lcFullFilename
		
		lnRetVal = 0
		lcFullFileName = IIF(EMPTY(tcFullFilename), "", ALLTRIM(tcFullFilename))
		lcScanFor = IIF(EMPTY(lcFullFilename), "!EMPTY(cn_classes.filenameex)", "ALLTRIM(cn_classes.filenameex) = lcFullFilename")
		SELECT cn_classes
		SET ORDER TO classname

		LOCATE
		SCAN FOR &lcScanFor
			lnRecNo = RECNO()
			lnLevel = 1
			lcParentKey = ""
			FOR lnLoop = 1 TO tnLevel
				lcParent = UPPER(cn_classes.parent)
				IF SEEK(lcParent, "cn_classes", "classname")
					IF lnLevel = 1
						lcParentKey = cn_classes.id
					ENDIF
					lnLevel = lnLevel + 1
					lcParent = UPPER(cn_classes.parent)
				ELSE
					EXIT
				ENDIF	
			ENDFOR
			IF lnLevel = tnLevel
				GO (lnRecNo)
				lnRetVal = lnRetVal + 1
				DIMENSION raChildClasses[lnRetVal,7]
				raChildClasses[lnRetVal,1] = cn_classes.id
				raChildClasses[lnRetVal,2] = ALLTRIM(cn_classes.classname)
				raChildClasses[lnRetVal,3] = ALLTRIM(cn_classes.parent)
				raChildClasses[lnRetVal,4] = lcParentKey
				raChildClasses[lnRetVal,5] = cn_classes.olepublic
				raChildClasses[lnRetVal,6] = cn_classes.startchar
				raChildClasses[lnRetVal,7] = ALLTRIM(cn_classes.filenameex)
			ELSE
				GO (lnRecNo)			
			ENDIF
		ENDSCAN
		
		RETURN lnRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetMaxClassGeneration()
	*-------------------------------------------------------
	*) Description: Returns the maximum number of genrations
	*)	 the classes have. For example, if every
	*)	class is currently a root class, this number would
	*)	be 1. If there were at least one child class this
	*)	would be 2. If there were at least one grandchild
	*)	class this would be 3, etc. Optionally, if 
	*)	tcFullFilename is passed only classes from a 
	*)	particular file will be considered.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetMaxClassGeneration(tcFullFilename)
		LOCAL lnRetVal, lnRecNo, lnLevel, lcParent, ;
			lcParentID, lcScanFor, lcFullFilename, ;
			lnMax
		
		lnRetVal = 1
		lcFullFileName = IIF(EMPTY(tcFullFilename), "", ALLTRIM(tcFullFilename))
		lcScanFor = IIF(EMPTY(lcFullFilename), "!EMPTY(cn_classes.filenameex)", "ALLTRIM(cn_classes.filenameex) = lcFullFilename")
		SELECT cn_classes
		SET ORDER TO

		LOCATE
		SCAN FOR &lcScanFor
			lnMax    = 1
			lnRecNo = RECNO()
			lcParentKey = ""
			DO WHILE .T.
				lcParent = UPPER(cn_classes.parent)
				IF SEEK(lcParent, "cn_classes", "classname")
					lnMax = lnMax + 1
				ELSE
					EXIT
				ENDIF	
			ENDDO
			lnRetVal = IIF(lnMax > lnRetVal, lnMax, lnRetVal)
			GO (lnRecNo)
		ENDSCAN
		
		RETURN lnRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetClassByClassID()
	*-------------------------------------------------------
	*) Description: Populates the raClass parameter
	*)	with an array of class information. Returns .f. if
	*)	no class is found. The array has the following
	*)	structure:
	*)
	*)	aClass[1] = Class id
	*)	aClass[2] = Class name
	*)	aClass[3] = Parent id (if any)
	*)	aClass[4] = Parent name
	*)	aClass[5] = Olepublic
	*)	aClass[6] = Startchar (where in the prg the define class is)
	*)	aClass[7] = Filename
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetClassByClassID(tcClassID, raClass)
		LOCAL llRetVal, lcParent
		
		llRetVal = SEEK(tcClassID, "cn_classes", "id")
		IF llRetVal
			DIMENSION raClass[7]
			raClass[1] = cn_classes.id
			raClass[2] = ALLTRIM(cn_classes.classname)
			raClass[3] = ""
			raClass[4] = ALLTRIM(cn_classes.parent)
			raClass[5] = cn_classes.olepublic
			raClass[6] = cn_classes.startchar
			raClass[7] = ALLTRIM(cn_classes.filenameex)

			lcParent = UPPER(cn_classes.parent)
			IF SEEK(lcParent, "cn_classes", "classname")
				raClass[3] = ALLTRIM(cn_classes.id)
			ENDIF
		ENDIF
		
		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetMethodByMethodID()
	*-------------------------------------------------------
	*) Description: Populates the raMethod parameter
	*)	with an array of method information. Returns .f. if 
	*)	no method is found. The array has the following
	*)	structure:
	*)
	*)	aMethod[1] = Method id
	*)	aMethod[2] = Method name
	*)	aMethod[3] = Class id 
	*)	aMethod[4] = Parameters
	*)	aMethod[5] = Protected
	*)	aMethod[6] = Hidden
	*)	aMethod[7] = Startchar
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetMethodByMethodID(tcMethodID, raMethod)
		LOCAL llRetVal
		
		llRetVal = SEEK(tcMethodID, "cn_methods", "id")
		IF llRetVal
			DIMENSION raMethod[7]
			raMethod[1] = cn_methods.id
			raMethod[2] = ALLTRIM(cn_methods.methname)
			raMethod[3] = cn_methods.classid
			raMethod[4] = ALLTRIM(cn_methods.parameters)
			raMethod[5] = cn_methods.protected
			raMethod[6] = cn_methods.hidden
			raMethod[7] = cn_methods.startchar
		ENDIF
		
		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	
	
	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetChildClasses()
	*-------------------------------------------------------
	*) Description: Returns an array of child classes 
	*)	(subclasses).
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetChildClasses(tcClassID, raChildClasses)
		LOCAL lnRetVal, lcParent
		
		lnRetVal = 0
		SELECT cn_classes
		SET ORDER TO classname
		IF SEEK(tcClassID, "cn_classes", "id")
			lcParent = ALLTRIM(cn_classes.classname)
			SCAN FOR cn_classes.parent = lcParent
				lnRetVal = lnRetVal + 1
				DIMENSION raChildClasses[lnRetVal,7]
				raChildClasses[lnRetVal,1] = cn_classes.id
				raChildClasses[lnRetVal,2] = ALLTRIM(cn_classes.classname)
				raChildClasses[lnRetVal,3] = ALLTRIM(cn_classes.parent)
				raChildClasses[lnRetVal,4] = tcClassID
				raChildClasses[lnRetVal,5] = cn_classes.olepublic
				raChildClasses[lnRetVal,6] = cn_classes.startchar
				raChildClasses[lnRetVal,7] = ALLTRIM(cn_classes.filenameex)
			ENDSCAN
		ENDIF
		
		RETURN lnRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetClassProperties()
	*-------------------------------------------------------
	*) Description: Returns an array of class properties
	*)	for a class
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetClassProperties(tcClassID, raClassProperties)
		LOCAL lnRetVal
		
		lnRetVal = 0
		SELECT cn_properties
		SET ORDER TO propname
		SCAN FOR cn_properties.classid = tcClassID
			lnRetVal = lnRetVal + 1
			DIMENSION raClassProperties[lnRetVal,6]
			raClassProperties[lnRetVal,1] = cn_properties.id
			raClassProperties[lnRetVal,2] = ALLTRIM(cn_properties.propname)
			raClassProperties[lnRetVal,3] = cn_properties.protected
			raClassProperties[lnRetVal,4] = cn_properties.hidden
			raClassProperties[lnRetVal,5] = cn_properties.value
			raClassProperties[lnRetVal,6] = cn_properties.startchar
		ENDSCAN
		
		RETURN lnRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetClassMethods()
	*-------------------------------------------------------
	*) Description: Returns an array of class methods
	*)	for a class
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetClassMethods(tcClassID, raClassMethods)
		LOCAL lnRetVal
		
		lnRetVal = 0
		SELECT cn_methods
		SET ORDER TO methname
		SCAN FOR cn_methods.classid = tcClassID
			lnRetVal = lnRetVal + 1
			DIMENSION raClassMethods[lnRetVal,6]
			raClassMethods[lnRetVal,1] = cn_methods.id
			raClassMethods[lnRetVal,2] = ALLTRIM(cn_methods.methname)
			raClassMethods[lnRetVal,3] = cn_methods.protected
			raClassMethods[lnRetVal,4] = cn_methods.hidden
			raClassMethods[lnRetVal,5] = cn_methods.parameters
			raClassMethods[lnRetVal,6] = cn_methods.startchar
		ENDSCAN
		
		RETURN lnRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  ZapClassCursors()
	*-------------------------------------------------------
	*) Description:  Cleans out all class cursors
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE ZapClassCursors()
		LOCAL lnSelected
		
		lnSelected = SELECT(0)
		IF !USED('cn_classes')
			SELECT cn_classes
			ZAP
		ENDIF

		IF !USED('cn_properties')
			SELECT cn_properties
			ZAP
		ENDIF

		IF !USED('cn_methods')
			SELECT cn_methods
			ZAP
		ENDIF
		
		SELECT (lnSelected)		

	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  WordsToArray()
	*-------------------------------------------------------
	*) Description:  Accepts a string and converts the string
	*)	into a one dimensional array. Each word in the string
	*)	will be in its own element in the array.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes: When converting a string of words to an array
	*$	the following rules need to be observed:
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE WordsToArray(tcWordString, raWordArray)
		LOCAL lnRetVal, lcWordString, lnBrackets, ;
			llInQuotes, lnCount, lcLetter, lcNewList
		
		DIMENSION raWordArray[1]
		lcWordString = ALLTRIM(tcWordString)
		
		*-- In its simplest form, STRTRAN() could be used
		*-- to convert each space in the string into a
		*-- carriage return and then use ALINES() to 
		*-- convert into an array. However if this was done
		*-- functions and array would be broken up for.
		*-- For example SUBSTR(lcSring, 1, 10) would become
		*-- 3 different words. Also, there are some 
		*-- special words that may not have spaces between
		*-- them for example lcName="CSession" should be
		*-- three words, not one. Finally, everything in
		*-- a string whould be a single word so
		*-- lcTitle = "Class Navigator" should also be
		*-- three words. Iterate through the string 
		*-- placing the CRs manually.
		lnBrackets = 0
		llInQuotes = .F.
		lcNewList  = ""
		
		FOR lnCount = 1 TO LEN(lcWordString)
			lcLetter = SUBSTR(lcWordString, lnCount, 1)
			DO CASE
				CASE lcLetter = "(" OR lcLetter = "[" AND NOT llInQuotes
					lnBrackets = lnBrackets + 1
					lcNewList = lcNewList + lcLetter
				CASE lcLetter = ")" OR lcLetter = "]" AND NOT llInQuotes
					lnBrackets = lnBrackets - 1
					lcNewList = lcNewList + lcLetter
				CASE lcLetter = '"' OR lcLetter = "'"
					llInQuotes = NOT llInQuotes
					lcNewList = lcNewList + lcLetter
				CASE lcLetter = " " AND lnBrackets = 0 AND NOT llInQuotes AND RIGHT(lcNewList,1) # CHR(13)
					lcNewList = lcNewList + CHR(13)
				CASE lcLetter = " " AND lnBrackets = 0 AND NOT llInQuotes AND RIGHT(lcNewList,1) = CHR(13)
					*-- Do nothing. Adding a CR here would create
					*-- an empty word.
				CASE lcLetter = "=" AND NOT llInQuotes
					lcNewList = lcNewList + IIF(RIGHT(lcNewList,1) # CHR(13), CHR(13), SPACE(0)) + lcLetter + CHR(13)
				OTHERWISE
					lcNewList = lcNewList + lcLetter
			ENDCASE
		ENDFOR
		
		*-- Now, use ALINES() to do all the work
		lnRetVal = ALINES(raWordArray, lcNewList)
		
		RETURN lnRetVal
		
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  ListToArray()
	*-------------------------------------------------------
	*) Description:  Accepts a comma delimited list and
	*)	converts each item in the list into an element in
	*)	an array.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROTECTED PROCEDURE ListToArray(tcWordList, raWordArray)
		LOCAL lnRetVal, lcWordList, lnCount, ;
			lnWords, lcLetter, lnBrackets, ;
			lcNewList
		
		lnBrackets = 0
		lcNewList = ""
		lcWordList = ALLTRIM(tcWordList)


		*-- This one is a bit tricky because a comma can
		*-- appear either between two words or in an
		*-- array definition. Argh. So, loop through each
		*-- letter in the string changing commas to
		*-- carriage returns unless the comma appears
		*-- between either () or [].
		FOR lnCount = 1 TO LEN(lcWordList)
			lcLetter = SUBSTR(lcWordList, lnCount, 1)
			DO CASE
				CASE lcLetter = "(" OR lcLetter = "["
					lnBrackets = lnBrackets + 1
					lcNewList = lcNewList + lcLetter
				CASE lcLetter = ")" OR lcLetter = "]"
					lnBrackets = lnBrackets - 1
					lcNewList = lcNewList + lcLetter
				CASE lcLetter = "," AND lnBrackets = 0
					lcNewList = lcNewList + CHR(13)
				CASE lcLetter = " "
					*-- Don't do anything.
				OTHERWISE
					lcNewList = lcNewList + lcLetter
			ENDCASE
		ENDFOR
		
		*-- We should now have a string with the
		*-- CHR(13)'s in place.		
		*-- Now, use ALINES() to do all the work
		lnRetVal = ALINES(raWordArray, lcNewList)
		
		RETURN lnRetVal
		
	ENDPROC
	*///////////////////////////////////////////////////////	


	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetWordInString()
	*-------------------------------------------------------
	*) Description: Returns a word from a string of words
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Character
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetWordInString(tcWordString, tnGetWord)
		LOCAL lcRetVal, lcWordString, laWordArray[1], ;
			lnWords
		
		lcRetVal = ""
		lnWords = THIS.WordsToArray(tcWordString, @laWordArray)
		IF lnWords >= tnGetWord
			lcRetVal = ALLTRIM(laWordArray[tnGetWord])
		ENDIF
		
		RETURN lcRetVal
		
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  LineNumberToCharNumber()
	*-------------------------------------------------------
	*) Description: Accepts a line number and determines 
	*)	which character number in the aClassFile array that 
	*)	line represents. This is used primarily because 
	*) 	MODIFY COMMAND has an option to set the starting
	*)	position of the cursor. This starting position is
	*)	based on the character number, not the line number.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Numeric
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE LineNumberToCharNumber(tnLine)
		LOCAL lnRetVal, lnCount
		
		lnRetVal = 0
		FOR lnCount = 1 TO tnLine - 1
			*-- Get the length of the line and add +2 to account
			*-- for the CR + LF
			lnRetVal = lnRetVal + LEN(THIS.aClassFile[lnCount]) + 2
		ENDFOR
		
		*-- The current value of lnRetVal will place the cursor
		*-- at the very end of the line before tnLine. To make
		*-- it appear at the beginning of the next line add
		*-- one more to it.
		lnRetVal = lnRetVal + 1
		
		RETURN lnRetVal
		
	ENDPROC
	*///////////////////////////////////////////////////////	

	*---------------- Location Section ---------------------
	*} Library: CCLASSPARSER.PRG
	*} Class:   CProgrammaticClassParser
	*} Method:  GetErrorString()
	*-------------------------------------------------------
	*) Description: Returns true if there is text in 
	*)	the cErrorString property, copies the string
	*)	to the parameter and clears the string.
	*------------------- Usage Section ---------------------
	*$ Scope:		
	*$ Parameters:  
	*$ Usage:		
	*$ Returns:		Logical
	*$ Notes:		
	*-------------------------------------------------------
	*!	Copyright:
	*!		Flash Creative Management, Inc., 1999
	*!		Author: Michael G. Emmons
	*!		Architect: Michael G. Emmons
	*-------------------------------------------------------
	PROCEDURE GetErrorString(rcErrorString)
		LOCAL llRetVal
		
		llRetVal = !EMPTY(THIS.cErrorString)
		rcErrorString = THIS.cErrorString
		THIS.cErrorString = ""
	
		RETURN llRetVal
	ENDPROC
	*///////////////////////////////////////////////////////	

ENDDEFINE

